@@ -15,7 +15,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin ST, Fifth Floor, Boston, MA  02110-1301  USA
 
-import gtk, gnome, gobject
+import gtk, gobject
 import gu
 import inputwidgets
 from specialwidgets import RandomTransposeDialog
@@ -29,6 +29,8 @@ import sys
 import signal
 import cfg
 import lessonfile
+import osutils
+
 
 class Teacher(cfg.ConfigUtils):
     def __init__(self, exname, app):
@@ -37,6 +39,10 @@ class Teacher(cfg.ConfigUtils):
         self.q_status = const.QSTATUS_NO
         self.m_statistics = None
         self.m_timeout_handle = None
+        self.m_P = None
+        # The file name, not a file object
+        self.m_lessonfile = None
+        self.m_question = None
     def get_instrument_config(self, num):
         if self.get_bool('config/override_default_instrument'):
             instr_low = self.get_int('config/lowest_instrument')
@@ -70,19 +76,6 @@ class Teacher(cfg.ConfigUtils):
         soundcard.synth.stop()
         if self.m_statistics:
             self.m_statistics.save_data()
-
-class LessonbasedTeacher(Teacher):
-    def __init__(self, exname, app):
-        Teacher.__init__(self, exname, app)
-        self.m_P = None
-        # The file name, not a file object
-        self.m_lessonfile = None
-        self.m_question = None
-    def is_test_complete(self):
-        """
-        Shared between harmonic and melodic interval.
-        """
-        return self.m_test_idx == len(self.m_test_questions) - 1
     def exit_test_mode(self):
         """
         Shared between harmonic and melodic interval.
@@ -111,17 +104,13 @@ class LessonbasedTeacher(Teacher):
         self.m_P = None
         try:
             self.m_P = self.lessonfileclass(self.m_lessonfile, self)
+        # We don't have to check for LessonfileExceptions here because
+        # the global exception hook will catch them.
         except IOError:
             self.m_app.m_ui.display_error_message(
 """There was an IOError while trying to parse a lesson file. The cause was probably you deleting or renaming a lesson file. Please don't do this while Solfege is running. The program is still a little fragile. 
 
-You really should reastart the program now.""")
-            return
-        except mpd.MpdException, e:
-            self.m_app.m_ui.display_exception_message(e, msg="Error parsing lessonfile '%s':" % self.m_lessonfile)
-            return
-        except lessonfile.LessonfileException, e:
-            self.m_app.m_ui.display_exception_message(e, __file__)
+You really should restart the program now.""")
             return
         if filter(lambda q: q['music'].m_musictype == 'cmdline',
                   self.m_P.m_questions):
@@ -131,8 +120,14 @@ You really should reastart the program now.""")
     def check_askfor(self):
         if self.m_custom_mode:
             self.set_list('ask_for_names', range(len(self.m_P.get_unique_cnames())))
+    def play_tonic(self):
+        """
+        Play the tonic of the question, if defined.
+        """
+        if 'tonic' in self.m_P.get_question():
+            self.m_P.play_question(None, 'tonic')
 
-class MelodicIntervalTeacher(LessonbasedTeacher):
+class MelodicIntervalTeacher(Teacher):
     """
     Base class for interval exercises where
     you can have more than one interval.
@@ -143,7 +138,7 @@ class MelodicIntervalTeacher(LessonbasedTeacher):
     ERR_PICKY = 1
     ERR_CONFIGURE = 2
     def __init__(self, exname, app):
-        LessonbasedTeacher.__init__(self, exname, app)
+        Teacher.__init__(self, exname, app)
         self.m_tonika = None
         self.m_question = []
     def new_question(self, L, H):
@@ -159,8 +154,8 @@ class MelodicIntervalTeacher(LessonbasedTeacher):
             self.m_timeout_handle = None
 
         if self.m_app.m_test_mode:
-            self.m_test_idx += 1
-            self.m_question = [self.m_test_questions[self.m_test_idx]]
+            self.m_P.next_test_question()
+            self.m_question = [self.m_P.m_test_questions[self.m_P.m_test_idx]]
             #FIXME use tone pitch range from preferences window.
             self.m_tonika = mpd.MusicalPitch()
             self.m_tonika.randomize("f", "f'")
@@ -244,8 +239,8 @@ class Gui(gtk.VBox, cfg.ConfigUtils):
         self.g_win = window
 
         vbox = gtk.VBox()
-        vbox.set_spacing(gnome.ui.PAD)
-        vbox.set_border_width(gnome.ui.PAD)
+        vbox.set_spacing(gu.PAD)
+        vbox.set_border_width(gu.PAD)
         vbox.show()
 
         self.practise_box = gtk.VBox()
@@ -259,7 +254,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils):
         vbox.pack_start(self.action_area, False)
 
         self.config_box = gtk.VBox()
-        self.config_box.set_border_width(gnome.ui.PAD)
+        self.config_box.set_border_width(gu.PAD)
         self.config_box.show()
         if no_notebook:
             self.pack_start(vbox)
@@ -291,9 +286,12 @@ class Gui(gtk.VBox, cfg.ConfigUtils):
         self.g_cancel_test.hide()
         self.g_win.exit_test_mode()
     def set_lesson_heading(self, txt):
-        self.g_lesson_heading.set_text('<span size="large"><b>%s</b></span>' % txt)
-        self.g_lesson_heading.set_use_markup(True)
-        self.g_lesson_heading.show()
+        if txt:
+            self.g_lesson_heading.set_text('<span size="large"><b>%s</b></span>' % txt)
+            self.g_lesson_heading.set_use_markup(True)
+            self.g_lesson_heading.show()
+        else:
+            self.g_lesson_heading.hide()
     def on_start_practise(self):
         """Dummy function that child classes can override.
         """
@@ -327,7 +325,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils):
         self.g_notebook.connect('switch_page', self.on_switch_page)
     def on_switch_page(self, notebook, obj, pagenum):
         if pagenum == 2:
-            if isinstance(self.m_t, LessonbasedTeacher):
+            if isinstance(self.m_t, Teacher):
                 if self.m_t.m_P:
                     self.g_statview.update()
                 else:
@@ -336,7 +334,7 @@ class Gui(gtk.VBox, cfg.ConfigUtils):
                 self.g_statview.update()
     def _add_auto_new_question_gui(self, box):
         hbox = gu.bHBox(box, False)
-        hbox.set_spacing(gnome.ui.PAD_SMALL)
+        hbox.set_spacing(gu.PAD_SMALL)
         adj = gtk.Adjustment(0, 0, 10, 0.1, 1, 1)
         spin = gu.nSpinButton(self.m_exname, 'seconds_before_new_question',
                        adj)
@@ -353,6 +351,64 @@ class Gui(gtk.VBox, cfg.ConfigUtils):
         hbox.pack_start(label, False)
         spin.set_sensitive(b.get_active())
         hbox.pack_start(spin, False)
+    def _lessonfile_exception(self, exception, sourcefile, lineno):
+        msg = _('Please check question number %(idx)i in the lesson file "%(lf)s".\n\n') % {'idx': self.m_t.m_P._idx+1, 'lf': self.m_t.m_P.m_filename}
+        msg += _('The exception was caught in\n"%(filename)s", line %(lineno)i.') % {'filename': sourcefile, 'lineno': lineno}
+        m = gtk.MessageDialog(self.g_win, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
+                              gtk.BUTTONS_CLOSE, str(exception))
+        m.format_secondary_text(msg)
+        m.set_title(sys.exc_info()[0].__name__)
+        m.run()
+        m.destroy()
+    def _mpd_exception(self, exception, sourcefile, lineno):
+        msg = _('Failed to parse the music for question number %(idx)i in the lesson file "%(lf)s".\n\n') % {'idx': self.m_t.m_P._idx+1, 'lf': self.m_t.m_P.m_filename}
+        msg += _('The exception was caught in\n"%(filename)s", line %(lineno)i.') % {'filename': sourcefile, 'lineno': lineno}
+        m = gtk.MessageDialog(self.g_win, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR,
+                              gtk.BUTTONS_CLOSE, str(exception))
+        m.format_secondary_text(msg)
+        m.set_title(sys.exc_info()[0].__name__)
+        m.run()
+        m.destroy()
+    def run_exception_handled(self, method, *args, **kwargs):
+        """
+        Call method() and catch exceptions with standard_exception_handler.
+        """
+        import traceback
+        try:
+            return method(*args, **kwargs)
+        except lessonfile.LessonfileException, e:
+            t = traceback.extract_stack()
+            self._lessonfile_exception(e, t[-2][0], t[-2][1])
+        except mpd.MpdException, e:
+            t = traceback.extract_stack()
+            self._mpd_exception(e, t[-2][0], t[-2][1])
+        except Exception, e:
+            t = traceback.extract_stack()
+            if not self.standard_exception_handler(e, __file__):
+                raise e
+    def standard_exception_handler(self, e, sourcefile, cleanup_function=lambda: False):
+        if self.m_t.m_app.m_options.disable_exception_handler:
+            import traceback
+            print >> sys.stderr, "standard_exception_handler"
+            print >> sys.stderr, "=========================="
+            traceback.print_exc(file=sys.stderr)
+            print >> sys.stderr, "=========================="
+            print >> sys.stderr, "The above exception was raised in %s:%i" % (sourcefile, sys.exc_info()[2].tb_lineno)
+            return True
+        if isinstance(e, lessonfile.LessonfileException):
+            cleanup_function()
+            self._lessonfile_exception(e, sourcefile, sys.exc_info()[2].tb_lineno)
+            return True
+        elif isinstance(e, mpd.MpdException):
+            cleanup_function()
+            self._mpd_exception(e, sourcefile, sys.exc_info()[2].tb_lineno)
+            return True
+        elif isinstance(e, osutils.ExecutableDoesNotExist):
+            cleanup_function()
+            self.g_win.display_exception_message(e, sourcefile)
+            return True
+        return False
+
 
 class IntervalGui(Gui):
     """
@@ -366,7 +422,7 @@ class IntervalGui(Gui):
         self.g_flashbar = gu.FlashBar()
         self.g_flashbar.show()
         self.practise_box.pack_start(self.g_flashbar, False)
-        self.practise_box.set_spacing(gnome.ui.PAD)
+        self.practise_box.set_spacing(gu.PAD)
 
         self.g_new_interval = gu.bButton(self.action_area, _("_New interval"),
                                           self.new_question)
@@ -379,7 +435,7 @@ class IntervalGui(Gui):
         constructor
         """
         hbox = gu.bHBox(self.config_box, False)
-        hbox.set_spacing(gnome.ui.PAD_SMALL)
+        hbox.set_spacing(gu.PAD_SMALL)
         gu.bLabel(hbox, _("Input interface:"), False)
 
         combo = gtk.combo_box_new_text()
@@ -473,4 +529,64 @@ class LessonbasedGui(Gui):
             else:
                 self.g_random_transpose.set_text(str(self.m_t.m_P.header.random_transpose))
         dlg.destroy()
+    def show_answer(self, widget=None):
+        """
+        Show the answer in the g_music_displayer if we have one, if not
+        use a new window.
+        """
+        if 'vmusic' in self.m_t.m_P.get_question():
+            varname = 'vmusic'
+        else:
+            varname = 'music'
+        self.display_music(varname)
+    def display_music(self, varname):
+        """
+        Display the music in the variable named by varname from
+        the currently selected question. This method will handle
+        the normal mpd and lessonfile exceptions.
+        """
+        try:
+            if self.m_t.m_P.header.have_music_displayer:
+                fontsize = self.get_int('config/feta_font_size=20')
+                self.g_music_displayer.display(self.m_t.m_P.get_music(varname), fontsize)
+            else:
+                self.g_win.display_in_musicviewer(self.m_t.m_P.get_music(varname))
+        except Exception, e:
+            if not self.standard_exception_handler(e, __file__):
+                raise e
+    def do_at_question_start_show_play(self):
+        """
+        This method is shared by idbyname and elembuilder, and possibly
+        other exercises later. It will show and/or play music based on
+        the header.at_question_start  variable.
+        """
+        if self.m_t.m_P.header.at_question_start == 'show':
+            self.show_answer()
+        elif self.m_t.m_P.header.at_question_start == 'play':
+            self.m_t.m_P.play_question()
+            if 'cuemusic' in self.m_t.m_P.get_question():
+                self.display_music('cuemusic')
+        else:
+            self.m_t.m_P.play_question()
+            if 'show' in self.m_t.m_P.header.at_question_start \
+                and 'play' in self.m_t.m_P.header.at_question_start:
+                self.show_answer()
+            elif 'cuemusic' in self.m_t.m_P.get_question():
+                self.display_music('cuemusic')
+    def show_hide_at_question_start_buttons(self):
+        """
+        Show and hide g_play_music, g_repeat and g_display_music
+        depending on the content of header.at_question_start.
+        This method is used by at least idbyname and elembuilder.
+        """
+        if self.m_t.m_P.header.at_question_start == 'show':
+            self.g_play_music.show()
+            self.g_repeat.hide()
+        else:
+            self.g_play_music.hide()
+            self.g_repeat.show()
+        if self.m_t.m_P.header.at_question_start == 'play':
+            self.g_display_music.show()
+        else:
+            self.g_display_music.hide()
 
