@@ -30,7 +30,11 @@ from osutils import *
 def linux_play_wav_file(filename):
     run_external_program(cfg.get_string('sound/wav_player'), "", filename)
 
+def linux_play_midi_file(filename):
+    run_external_program(cfg.get_string('sound/midi_player'), "", filename)
+
 soundcard.play_wav_file = linux_play_wav_file
+soundcard.play_midi_file = linux_play_midi_file
 
 def check_rcfile():
     """See default.config for rcfileversion values, meanings and
@@ -199,7 +203,11 @@ It is also possible that the OS sound setup is incorrect.
         self.m_ui.show_help_on_current()
         return module
     def practise_lesson_id(self, lesson_id):
-        module = self._practise_lesson_id(lesson_id)
+        try:
+            module = self._practise_lesson_id(lesson_id)
+        except lessonfile.LessonfileException, e:
+            self.m_ui.display_error_message(str(e))
+            module = self.lessonfile_manager.get(lesson_id, 'module')
         if not self.m_teachers[module].m_P:
             for n in self.m_ui.box_dict[module].action_area.get_children():
                 n.set_sensitive(False)
