@@ -17,7 +17,7 @@
 
 import sys
 import os
-import gobject, gtk, gnome, gnome.ui
+import gobject, gtk
 import gu, mpd
 import configureoutput
 import notenamespinbutton
@@ -354,17 +354,11 @@ class ConfigWindow(gtk.Dialog, cfg.ConfigUtils):
         gu.bButton(category_vbox, _("_Apply changes and play test sound"), self.on_apply_and_play_test_sound)
 
     def test_wav_player(self, w):
-        try:
-            osutils.run_external_program(self.g_wav_player.child.get_text(),
+        osutils.run_external_program(self.g_wav_player.child.get_text(),
                  'lesson-files/share', 'fifth-small-220.00.wav')
-        except osutils.RunExternalFailed, e:
-                self.m_app.m_ui.display_error_message(str(e))
     def test_midi_player(self, w):
-        try:
-            osutils.run_external_program(self.g_midi_player.child.get_text(),
+        osutils.run_external_program(self.g_midi_player.child.get_text(),
                         'lesson-files/share', 'fanfare.midi')
-        except osutils.RunExternalFailed, e:
-                self.m_app.m_ui.display_error_message(str(e))
     def set_gui_from_config(self):
         if self.get_string("sound/type") == "fake-synth":
             self.g_fakesynth_radio.set_active(True)
