@@ -202,6 +202,7 @@ class Lexer:
             src = unicode(src, m.groups()[0], errors="replace")
         else:
             src = unicode(src, "UTF-8", errors="replace")
+        src = src.replace("\r", "\n")
         self.m_src = src
         pos = 0
         lineno = 0
@@ -275,6 +276,9 @@ class Dataparser:
         self.functions = function_dict.copy()
         self.header = {}
         self.questions = []
+        # Each block type will have a list in blocklists,
+        # for example self.blocklists['element'] = []
+        self.blocklists = {}
         self.context = self.globals
     def parse_file(self, filename):
         """We always construct a new parser if we want to parse another
@@ -300,6 +304,9 @@ class Dataparser:
             self.assignment()
         elif self._lexer.peek_type(1) == '{':
             self.block()
+        elif self._lexer.peek_type(1) == 'NAME' \
+                and self._lexer.peek_type(2) == '{':
+            self.named_block()
         elif self._lexer.peek_type() == 'NAME' \
                 and self._lexer.peek_string() == 'include' \
                 and self._lexer.peek_type(1) == '(':
@@ -448,6 +455,28 @@ class Dataparser:
                 if not (n in self.context):
                     self.context[n] = self.globals[n]
         self.context = self.globals
+    def named_block(self):
+        blocktype = self._lexer.scan('NAME')
+        name = self._lexer.scan('NAME')
+        #FIXME right now named_block is reserved to element blocks, but
+        # I hope to move other blocks here too. Or at least questions should
+        # use self.blocklists, I think.
+        assert blocktype == 'element'
+        if blocktype not in self.blocklists:
+            self.blocklists[blocktype] = []
+        elem = dict()
+        # We must add the name of the block to the global name space since
+        # it will be referred from other blocks.
+        self.globals[name] = elem
+        # And they have to be added to the list of blocks because we may
+        # need to access all blocks of a certain type.
+        self.blocklists[blocktype].append(elem)
+        elem['name'] = name
+        self._lexer.scan('{')
+        self.context = elem
+        self.assignmentlist()
+        self._lexer.scan("}")
+        self.context = self.globals
     def lookup_name(self, name):
         if name in self.context:
             return self.context[name]
