@@ -1,5 +1,5 @@
 # GNU Solfege - ear training for GNOME
-# Copyright (C) 2005 Tom Cato Amundsen
+# Copyright (C) 2005, 2006 Tom Cato Amundsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@ import gtk
 from docviewer import DocViewer
 import history
 import utils
+import os
 
 class HelpBrowser(gtk.Window):
     def __init__(self, app):
@@ -56,42 +57,60 @@ class HelpBrowser(gtk.Window):
         self.show_docfile('index.html')
     def refresh(self, action):
         self.m_history.set_adj_of_current(self.g_docviewer.get_vadjustment().get_value())
-        self.handle_href(self.m_history.get_current()[0])
+        self.show_docfile(*self.m_history.get_current()[0].split("#"))
         while gtk.events_pending(): gtk.main_iteration()
-        self.g_docviewer.get_vadjustment().set_value(self.m_history.get_current()[1])
+        adj = self.m_history.get_current()[1]
+        if not adj:
+            adj = 0.0
+        self.g_docviewer.get_vadjustment().set_value(adj)
     def history_back(self, action):
         self.m_history.set_adj_of_current(self.g_docviewer.get_vadjustment().get_value())
         self.m_history.back()
         self.m_history.lock()
-        self.handle_href(self.m_history.get_current()[0])
+        self.show_docfile(*self.m_history.get_current()[0].split("#"))
         self.m_history.unlock()
         while gtk.events_pending(): gtk.main_iteration()
-        self.g_docviewer.get_vadjustment().set_value(self.m_history.get_current()[1])
+        adj = self.m_history.get_current()[1]
+        if not adj:
+            adj = 0.0
+        self.g_docviewer.get_vadjustment().set_value(adj)
     def history_forward(self, action):
         self.m_history.set_adj_of_current(self.g_docviewer.get_vadjustment().get_value())
         self.m_history.forward()
         self.m_history.lock()
-        self.handle_href(self.m_history.get_current()[0])
+        self.show_docfile(*self.m_history.get_current()[0].split("#"))
         self.m_history.unlock()
         while gtk.events_pending(): gtk.main_iteration()
-        self.g_docviewer.get_vadjustment().set_value(self.m_history.get_current()[1])
+        adj = self.m_history.get_current()[1]
+        if not adj:
+            adj = 0.0
+        self.g_docviewer.get_vadjustment().set_value(adj)
     def handle_href(self, filename):
-        fn = filename.split("#")[0]
-        anchor = "#".join(filename.split("#")[1:])
-        if fn.startswith('solfege:'):
+        if filename.startswith('solfege:'):
             self.m_app.handle_solfege_href(utils.Url(filename))
             self.m_app.m_ui.present()
             return
-        #print "FIXME not handling anchors"
+        fn = filename.split("#")[0]
+        anchor = "#".join(filename.split("#")[1:])
         # If history is locked, then we are probably called from the back
         # or forward functions, and they handle adjustment themselves.
         if not self.m_history.m_lock:
             self.m_history.set_adj_of_current(self.g_docviewer.get_vadjustment().get_value())
-        self.m_history.add(filename)
+        saved_document_wd = self.g_docviewer.m_htmlwidget.m_document_wd
         self.g_docviewer.read_docfile(fn, anchor)
+        # Find the absolute path to the file filename
+        absname = os.path.join(os.getcwd(),
+            saved_document_wd, self.g_docviewer.m_loaded_file)
+        self.m_history.add(absname)
     def show_docfile(self, filename, anchor=None):
-        self.m_history.add(filename)
-        self.g_docviewer.read_docfile(filename, anchor)
+        """
+        Load a html file relative to the help directory for the current
+        language, for example help/C or help/no.
+        """
+        absname = os.path.join(os.getcwd(), "help",
+            self.g_docviewer.m_language, filename)
+        self.m_history.add(absname)
+        self.g_docviewer.read_docfile(absname, anchor)
 
 if __name__ == '__main__':
     import sys
