@@ -15,7 +15,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin ST, Fifth Floor, Boston, MA  02110-1301  USA
 
-import gtk, gnome, gobject
+import gtk, gobject
 import gu
 from specialwidgets import QuestionNameButtonTable, QuestionNameCheckButtonTable
 import abstract, const, mpd, mpd.musicdisplayer
@@ -27,12 +27,12 @@ import soundcard
 import osutils
 
 
-class Teacher(abstract.LessonbasedTeacher):
+class Teacher(abstract.Teacher):
     OK = 0
     ERR_PICKY = 1
     ERR_NO_QUESTION = 2
     def __init__(self, exname,  app):
-        abstract.LessonbasedTeacher.__init__(self, exname, app)
+        abstract.Teacher.__init__(self, exname, app)
         self.lessonfileclass = lessonfile.IdByNameLessonfile
         self.m_statistics = statistics.LessonStatistics(self)
     def enter_test_mode(self):
@@ -98,24 +98,25 @@ class Gui(abstract.LessonbasedGui):
         self.g_flashbar = gu.FlashBar()
         self.g_flashbar.show()
         self.practise_box.pack_start(self.g_flashbar, False)
-        self.practise_box.set_spacing(gnome.ui.PAD)
+        self.practise_box.set_spacing(gu.PAD)
 
         self.g_new = gu.bButton(self.action_area, _("_New"), self.new_question)
         self.g_play_music = gu.bButton(self.action_area, _("P_lay music"),
-            lambda w: self.m_t.m_P.play_question())
+            lambda w: self.run_exception_handled(self.m_t.m_P.play_question))
         self.g_display_music = gu.bButton(self.action_area, _("_Display music"),
-            self.show_music)
+            self.show_answer)
         self.g_repeat = gu.bButton(self.action_area, _("_Repeat"),
-                                   self.repeat_question)
-        self.g_repeat_arpeggio = gu.bButton(self.action_area,
-              _("Repeat _arpeggio"), self.repeat_question_arpeggio)
-        self.g_repeat_slowly = gu.bButton(self.action_area,
-              _("Repeat _slowly"), self.repeat_question_slowly)
+            lambda w: self.run_exception_handled(self.m_t.m_P.play_question))
+        self.g_repeat_arpeggio = gu.bButton(self.action_area, 
+            _("Repeat _arpeggio"), 
+            lambda w: self.run_exception_handled(self.m_t.m_P.play_question_arpeggio))
+        self.g_repeat_slowly = gu.bButton(self.action_area, _("Repeat _slowly"),
+              lambda w: self.run_exception_handled(self.m_t.m_P.play_question_slowly))
         self.g_give_up = gu.bButton(self.action_area, 
               _("_Give up"), self.give_up)
         self.g_give_up.set_sensitive(False)
 
-        self.g_show = gu.bButton(self.action_area, _("_Show"), self.show_music)
+        self.g_show = gu.bButton(self.action_area, _("_Show"), self.show_answer)
         self.g_show.set_sensitive(False)
         self.practise_box.show_all()
         ##############
@@ -143,27 +144,6 @@ class Gui(abstract.LessonbasedGui):
         def _f(varname):
             self.setup_action_area_buttons()
         self.add_watch('ask_for_names', _f)
-    def show_music(self, widget=None):
-        """
-        Show the answer in the g_music_displayer if we have one, if not
-        use a new window.
-        """
-        if 'vmusic' in self.m_t.m_P.get_question():
-            varname = 'vmusic'
-        else:
-            varname = 'music'
-        if self.m_t.m_P.header.have_music_displayer:
-            fontsize = self.get_int('config/feta_font_size=20')
-            self.g_music_displayer.display(self.m_t.m_P.get_music(varname), fontsize)
-        else:
-            self.g_win.display_in_musicviewer(self.m_t.m_P.get_music(varname))
-    def display_music(self, varname):
-        """
-        Display the music in the variable named by varname from
-        the currently selected question.
-        """
-        fontsize = self.get_int('config/feta_font_size=20')
-        self.g_music_displayer.display(self.m_t.m_P.get_music(varname), fontsize)
     def update_answer_buttons(self):
         if self.m_t.m_P:
             self.g_bb.initialize(self.m_t.m_P.header.fillnum,
@@ -217,16 +197,7 @@ class Gui(abstract.LessonbasedGui):
             self.g_repeat_slowly.show()
         else:
             self.g_repeat_slowly.hide()
-        if self.m_t.m_P.header.at_question_start == 'show':
-            self.g_play_music.show()
-            self.g_repeat.hide()
-        else:
-            self.g_play_music.hide()
-            self.g_repeat.show()
-        if self.m_t.m_P.header.at_question_start == 'play':
-            self.g_display_music.show()
-        else:
-            self.g_display_music.hide()
+        self.show_hide_at_question_start_buttons()
         if self.m_t.m_P.header.have_music_displayer:
             self.g_music_displayer.show()
             self.g_music_displayer.clear()
@@ -243,7 +214,7 @@ class Gui(abstract.LessonbasedGui):
             self.g_new.grab_focus()
             self.g_give_up.set_sensitive(False)
             if self.m_t.m_P.header.have_music_displayer:
-                self.show_music()
+                self.run_exception_handled(self.show_answer)
     def on_click(self, button, event=None):
         if not event:
             self.on_left_click(button)
@@ -265,16 +236,20 @@ class Gui(abstract.LessonbasedGui):
                     const.QSTATUS_SOLVED):
             self.g_flashbar.flash(_("You should try to guess before right-clicking."))
             return
-        if 'set' in self.m_t.m_P.get_question():
+        try:
+            if 'set' in self.m_t.m_P.get_question():
+                for question in self.m_t.m_P.m_questions:
+                    if question['set'] == self.m_t.m_P.get_question()['set'] \
+                        and question.get_cname() == button.get_data('cname'):
+                        self.m_t.m_P.play_question(question)
+                        return
             for question in self.m_t.m_P.m_questions:
-                if question['set'] == self.m_t.m_P.get_question()['set'] \
-                    and question.get_cname() == button.get_data('cname'):
+                if question.get_cname() == button.get_data('cname'):
                     self.m_t.m_P.play_question(question)
                     return
-        for question in self.m_t.m_P.m_questions:
-            if question.get_cname() == button.get_data('cname'):
-                self.m_t.m_P.play_question(question)
-                return
+        except Exception, e:
+            if not self.standard_exception_handler(e, __file__):
+                raise e
     def on_left_click(self, button):
         if self.m_t.q_status == const.QSTATUS_NO:
             if self.m_t.m_app.m_test_mode:
@@ -282,25 +257,29 @@ class Gui(abstract.LessonbasedGui):
             else:
                 self.g_flashbar.flash(_("Click 'New' to begin."))
             return
-        if self.m_t.q_status == const.QSTATUS_SOLVED:
-            if self.m_t.guess_answer(button.get_data('cname')):
-                self.g_flashbar.flash(_("Correct, but you have already solved this question"))
-            else:
-                self.g_flashbar.flash(_("Wrong, but you have already solved this question"))
-        elif self.m_t.q_status in (const.QSTATUS_NEW, const.QSTATUS_WRONG):
-            if self.m_t.guess_answer(button.get_data('cname')):
-                self.g_flashbar.flash(_("Correct"))
-                self.g_new.set_sensitive(True)
-                self.g_new.grab_focus()
-                self.g_show.set_sensitive(True)
-                self.g_give_up.set_sensitive(False)
-                if self.m_t.m_P.header.have_music_displayer:
-                    self.show_music()
-            else:
-                self.g_flashbar.flash(_("Wrong"))
-                if self.get_bool("config/auto_repeat_question_if_wrong_answer"):
-                    self.m_t.m_P.play_question()
-                self.g_give_up.set_sensitive(True)
+        try:
+            if self.m_t.q_status == const.QSTATUS_SOLVED:
+                if self.m_t.guess_answer(button.get_data('cname')):
+                    self.g_flashbar.flash(_("Correct, but you have already solved this question"))
+                else:
+                    self.g_flashbar.flash(_("Wrong, but you have already solved this question"))
+            elif self.m_t.q_status in (const.QSTATUS_NEW, const.QSTATUS_WRONG):
+                if self.m_t.guess_answer(button.get_data('cname')):
+                    self.g_flashbar.flash(_("Correct"))
+                    self.g_new.set_sensitive(True)
+                    self.g_new.grab_focus()
+                    self.g_show.set_sensitive(True)
+                    self.g_give_up.set_sensitive(False)
+                    if self.m_t.m_P.header.have_music_displayer:
+                        self.show_answer()
+                else:
+                    self.g_flashbar.flash(_("Wrong"))
+                    if self.get_bool("config/auto_repeat_question_if_wrong_answer"):
+                        self.m_t.m_P.play_question()
+                    self.g_give_up.set_sensitive(True)
+        except Exception, e:
+            if not self.standard_excpetion_handler(e, __file__):
+                raise e
     def new_question(self, widget=None):
         """
         The new button should be insensitive if we have no lesson file.
@@ -323,19 +302,7 @@ class Gui(abstract.LessonbasedGui):
         try:
             g = self.m_t.new_question()
             if g == self.m_t.OK:
-                if self.m_t.m_P.header.at_question_start == 'show':
-                    self.show_music()
-                elif self.m_t.m_P.header.at_question_start == 'play':
-                    self.m_t.m_P.play_question()
-                    if 'cuemusic' in self.m_t.m_P.get_question():
-                        self.display_music('cuemusic')
-                else:
-                    self.m_t.m_P.play_question()
-                    if 'show' in self.m_t.m_P.header.at_question_start \
-                        and 'play' in self.m_t.m_P.header.at_question_start:
-                        self.show_music()
-                    elif 'cuemusic' in self.m_t.m_P.get_question():
-                        self.display_music('cuemusic')
+                self.do_at_question_start_show_play()
                 self.g_repeat.set_sensitive(True)
                 self.g_repeat_slowly.set_sensitive(True)
                 self.g_repeat_arpeggio.set_sensitive(True)
@@ -350,26 +317,9 @@ class Gui(abstract.LessonbasedGui):
         except lessonfile.NoQuestionConfiguredException, e:
             self.g_flashbar.clear()
             self.g_flashbar.flash(_("You have to select some questions to practise."))
-        except lessonfile.LessonfileException, e:
-            exception_cleanup()
-            self.g_win.display_exception_message(e, __file__)
-        except mpd.MpdException, e:
-            exception_cleanup()
-            self.g_win.display_question_music_error_message(
-                self.m_t.m_P._idx, self.m_t.m_lessonfile,
-                e, __file__)
-        except osutils.RunExternalFailed, e:
-            exception_cleanup()
-            self.g_win.display_exception_message(e, __file__)
-    def repeat_question(self, *w):
-        self.g_bb.grab_focus_first_button()
-        self.m_t.m_P.play_question()
-    def repeat_question_arpeggio(self, *w):
-        self.g_bb.grab_focus_first_button()
-        self.m_t.m_P.play_question_arpeggio()
-    def repeat_question_slowly(self, *w):
-        self.g_bb.grab_focus_first_button()
-        self.m_t.m_P.play_question_slowly()
+        except Exception, e:
+            if not self.standard_exception_handler(e, __file__, exception_cleanup):
+                raise e
     def on_start_practise(self):
         self.m_t.m_custom_mode = self.get_bool('gui/expert_mode')
         for question in self.m_t.m_P.m_questions:
@@ -390,6 +340,7 @@ class Gui(abstract.LessonbasedGui):
         self.g_new.grab_focus()
         if not self.m_t.m_custom_mode:
             self.m_t.m_statistics.reset_session()
+        self.g_statview.g_heading.set_text(self.m_t.m_P.header.title)
         self.g_music_displayer.clear()
         if self.m_t.m_app.m_test_mode:
             gobject.timeout_add(const.SHORT_WAIT, lambda self=self:
