@@ -369,7 +369,7 @@ class MainWin(App, cfg.ConfigUtils):
             v = os.listdir(os.path.join(gethomedir.get_home_dir(), ".solfege", "learningtrees"))
         except OSError:
             v = []
-        if not v:
+        if not self.m_app.m_options.debug and not v:
             return
         actions = [('LearningTreesMenu', None, _('Learning tree')),
             ('DefaultTree', None, _("Default learning tree"),
